import * as i1 from '@angular/common';
import { DOCUMENT, isPlatformBrowser, CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, Directive, Component, ChangeDetectionStrategy, Inject, NgZone, HostBinding, ChangeDetectorRef, ElementRef, ViewChild, Input, Injectable, EventEmitter, SecurityContext, Renderer2, Output, ContentChildren, isDevMode, ViewChildren, forwardRef, Injector, PLATFORM_ID, Optional, ContentChild, NgModule, ComponentFactoryResolver, ApplicationRef } from '@angular/core';
import { DomSanitizer, BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { Subject, fromEvent, Observable, pipe, from, merge } from 'rxjs';
import { filter, takeUntil, concatMap, map, startWith, switchMap, tap } from 'rxjs/operators';
import { __awaiter, __rest } from 'tslib';
import { generateHTML, Editor, mergeAttributes } from '@tiptap/core';
import { Mention } from '@tiptap/extension-mention';
import { Link } from '@tiptap/extension-link';

const FadeInAnimation = trigger('fadeIn', [
    transition(':enter', [
        style({ opacity: 0 }),
        animate('150ms cubic-bezier(.13,1.14,1,.92)', style({ opacity: 1 })),
    ]),
    transition(':leave', [
        animate('150ms cubic-bezier(.13,1.14,1,.92)', style({ opacity: 0 }))
    ])
]);

const PopOverPopUpAnimation = trigger('popUp', [
    transition(':enter', [
        query('.popover-wrapper', [
            style({ opacity: 0, transform: 'translate(-50%, -50%)' }),
            animate('150ms cubic-bezier(.13,1.14,1,.92)', style({ opacity: 1, transform: '*' })),
        ])
    ]),
    transition(':leave', query('.popover-wrapper', [
        animate('150ms cubic-bezier(.13,1.14,1,.92)', style({ opacity: 0, transform: 'translate(-50%, -50%)' })),
    ])),
]);
const OverlayPopUpAnimation = trigger('popUp', [
    transition(':enter', [
        query('.dialog-wrapper', [
            style({ transform: 'translate(-50%, -30%)' }),
            animate('150ms cubic-bezier(.13,1.14,1,.92)', style({ transform: '*' })),
        ])
    ]),
    transition(':leave', query('.dialog-wrapper', [
        animate('150ms cubic-bezier(.13,1.14,1,.92)', style({ transform: 'translate(-50%, -30%)' })),
    ])),
]);

const TIP_DIALOG_DATA = new InjectionToken('DIALOG_DATA');
class DialogRef {
    /* tslint:enable */
    constructor(_component, appRef, dialogWrapperComponentRef, config) {
        this._component = _component;
        this.appRef = appRef;
        this.dialogWrapperComponentRef = dialogWrapperComponentRef;
        this.config = config;
        /* tslint:disable member-ordering*/
        this.subject$ = new Subject();
        this.result$ = this.subject$.asObservable();
        this._done = false;
    }
    get component() {
        return this._component;
    }
    get done() {
        return this._done;
    }
    get dialogConfig() {
        const copy = Object.assign({}, this.config);
        delete copy.data;
        return copy;
    }
    get dialogWrapperComponent() {
        return this.dialogWrapperComponentRef.component;
    }
    submit(data) {
        this.sendResult(data, 'success');
    }
    cancel() {
        this.sendResult(null, 'canceled');
    }
    sendResult(data, status) {
        if (this.done)
            return;
        this.appRef.detachView(this.dialogWrapperComponent.hostView);
        this.dialogWrapperComponent.destroy();
        this._done = true;
        this.subject$.next({ data, status });
        this.subject$.complete();
    }
}
// tslint:disable-next-line:directive-class-suffix
class DialogBaseClass {
    constructor() {
        this.destroy$ = new Subject();
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            fromEvent(this.document, 'keydown').pipe(filter(e => e.key === 'Escape'), takeUntil(this.destroy$)).subscribe(() => this.ngZone.run(() => this.closeDialog()));
        });
    }
    ngOnDestroy() {
        this.destroy$.next(true);
        this.destroy$.complete();
    }
    closeDialog() {
        if (!this.dialogRef.dialogConfig.autoClose)
            return;
        this.dialogRef.cancel();
    }
}
DialogBaseClass.decorators = [
    { type: Directive }
];

// @dynamic
class DialogComponent extends DialogBaseClass {
    constructor(data, document, dialogRef, ngZone) {
        super();
        this.data = data;
        this.document = document;
        this.dialogRef = dialogRef;
        this.ngZone = ngZone;
        this.position = {};
        this.position = this.calculateStyle();
    }
    calculateStyle() {
        const config = this.dialogRef.dialogConfig;
        return {
            width: config.width,
            maxWidth: config.maxWidth,
            top: config.position === 'top' ? '20%' : '50%',
        };
    }
}
DialogComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-dialog',
                template: `
    <div class="overlay" (click)="closeDialog()" [ngStyle]="{backgroundColor: dialogRef.dialogConfig.backdropColor}"></div>
    <div class="dialog-wrapper" [ngStyle]="position">
      <ng-container *ngIf="dialogRef.component" [ngComponentOutlet]="dialogRef.component"></ng-container>
    </div>
  `,
                animations: [FadeInAnimation, OverlayPopUpAnimation],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`
    .overlay {
      position: fixed;
      top: 0;
      bottom: 0;
      right: 0;
      left: 0;
      z-index: 2000;
    }

    .dialog-wrapper {
      left: 50%;
      position: absolute;
      transform: translate(-50%, -50%);
      border-radius: 5px;
      padding: 10px;
      z-index: 2000;
      background-color: var(--tip-background-color);
    }
  `]
            },] }
];
DialogComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [TIP_DIALOG_DATA,] }] },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: DialogRef },
    { type: NgZone }
];
DialogComponent.propDecorators = {
    calculateStyle: [{ type: HostBinding, args: ['@fadeIn',] }, { type: HostBinding, args: ['@popUp',] }]
};

const findExtension = (editor, name) => {
    return editor.extensionManager.extensions.find(e => e.name === name);
};
const ɵ0$1 = findExtension;
const getHeadingsExtension = (editor) => {
    return findExtension(editor, 'heading');
};
const sleep = (ms) => new Promise(resolve => setTimeout(resolve, ms));
const fromEditorEvent = (editor, event, once = false) => {
    return new Observable((observer) => {
        const callback = (...params) => {
            observer.next(...params);
            if (once)
                editor.off(event, callback);
        };
        editor.on(event, callback);
        return () => {
            editor.off(event, callback);
        };
    });
};
const asyncFilter = (predicate) => {
    let count = 0;
    return pipe(
    // Convert the predicate Promise<boolean> to an observable (which resolves the promise,
    // Then combine the boolean result of the promise with the input data to a container object
    concatMap((data) => {
        return from(predicate(data, count++))
            .pipe(map((isValid) => ({ filterResult: isValid, entry: data })));
    }), 
    // Filter the container object synchronously for the value in each data container object
    filter(data => data.filterResult), 
    // remove the data container object from the observable chain
    map(data => data.entry));
};
const getSelectedTextPosition = () => {
    const selection = document.getSelection();
    if (!selection)
        return null;
    const range = selection.getRangeAt(0);
    return range.getBoundingClientRect();
};
const getLinkFromCursorPosition = (editor) => {
    const link = editor.getAttributes('link');
    return link && link.href ? link.href : null;
};
const getLinkDOMFromCursorPosition = ({ state: { selection: { from } }, view }, link = null) => {
    const dom = view.domAtPos(from);
    let selector = 'a';
    if (link)
        selector = `a[href="${link}"]`;
    return dom.node.parentElement.closest(selector);
};
const getSelectedEditorTextPosition = ({ state, view }) => {
    const { from, to } = state.selection;
    const start = view.coordsAtPos(from);
    const end = view.coordsAtPos(to);
    return {
        start, end,
        topCenter: {
            y: start.top,
            x: start.left + (end.left - start.left) / 2
        }
    };
};
const topCenterOfRect = (rect) => {
    return {
        y: rect.y,
        x: rect.x + rect.width / 2
    };
};
const getDuplicates = (list, getKey) => {
    const hashMap = {};
    for (const item of list) {
        const key = getKey(item);
        if (!(key in hashMap))
            hashMap[key] = [];
        hashMap[key].push(item);
    }
    const duplicates = {};
    for (const hashKey of Object.keys(hashMap)) {
        if (hashMap[hashKey].length > 1)
            duplicates[hashKey] = hashMap[hashKey];
    }
    return Object.keys(duplicates).length > 0 ? duplicates : null;
};
const isObject = (o) => typeof o === 'object' && !Array.isArray(o);
const deepMerge = (target, source) => {
    const targetCopy = Object.assign({}, target);
    for (const key of Object.keys(source)) {
        if (isObject(source[key])) {
            const newTarget = isObject(target[key]) ? target[key] : {};
            targetCopy[key] = deepMerge(newTarget, source[key]);
        }
        else {
            targetCopy[key] = source[key];
        }
    }
    return targetCopy;
};

// @dynamic
class PopoverComponent extends DialogBaseClass {
    constructor(data, document, dialogRef, ngZone, cd, element) {
        super();
        this.data = data;
        this.document = document;
        this.dialogRef = dialogRef;
        this.ngZone = ngZone;
        this.cd = cd;
        this.element = element;
        this.style = {};
        this.style = this.calculateStyle();
    }
    ngOnInit() {
        super.ngOnInit();
        this.ngZone.runOutsideAngular(() => __awaiter(this, void 0, void 0, function* () {
            yield sleep(1000);
            fromEvent(this.document, 'click')
                .pipe(takeUntil(this.destroy$), filter(e => !this.element.nativeElement.contains(e.target)))
                .subscribe(() => this.ngZone.run(() => this.dialogRef.cancel()));
        }));
    }
    ngAfterViewInit() {
        const position = this.popover.nativeElement.getBoundingClientRect();
        const window = this.document.defaultView;
        if (!window)
            return;
        const windowWidth = window.innerWidth;
        const overflowRight = (position.x + position.width) - windowWidth;
        const updateStyles = {};
        if (position.x < 0) {
            updateStyles.transform = `translate(calc(-50% + ${Math.abs(position.x - 5)}px), -100%)`;
        }
        else if (overflowRight > 0) {
            updateStyles.transform = `translate(calc(-50% - ${overflowRight + 5}px), -100%)`;
        }
        if (Object.keys(updateStyles).length > 0) {
            this.style = Object.assign(Object.assign({}, this.style), updateStyles);
            this.cd.detectChanges();
        }
    }
    calculateStyle() {
        const config = this.dialogRef.dialogConfig;
        return {
            'top.px': config.position.y - 10,
            'left.px': config.position.x,
        };
    }
}
PopoverComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-popover',
                template: `
    <div class="popover-wrapper" [ngStyle]="style" #popover>
      <ng-container *ngIf="dialogRef.component" [ngComponentOutlet]="dialogRef.component"></ng-container>
      <div class="hide-arrow"></div>
      <div class="arrow"></div>
    </div>
  `,
                animations: [PopOverPopUpAnimation],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`
    .popover-wrapper {
      border-radius: 5px;
      background-color: var(--tip-background-color);
      position: fixed;
      transform: translate(-50%, -100%);
      box-shadow: 0 1px 3px 1px rgba(60, 64, 67, .15);
      padding: 11px;
      z-index: 2000;
    }

    .hide-arrow {
      position: absolute;
      width: 30px;
      height: 10px;
      transform: translate(-50%, -100%);
      top: 100%;
      left: 50%;
      background-color: var(--tip-background-color);
      z-index: 1;
    }

    .arrow {
      position: absolute;
      top: 100%;
      left: 50%;
      width: 0;
      height: 0;
      box-shadow: 1px 1px 3px 1px rgba(60, 64, 67, .15);
      border: 7px solid var(--tip-background-color);
      transform: rotate(45deg) translate(-50%, -50%);
      transform-origin: 0 0;
      z-index: 0;
    }

  `]
            },] }
];
PopoverComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [TIP_DIALOG_DATA,] }] },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: DialogRef },
    { type: NgZone },
    { type: ChangeDetectorRef },
    { type: ElementRef }
];
PopoverComponent.propDecorators = {
    popover: [{ type: ViewChild, args: ['popover',] }],
    ngAfterViewInit: [{ type: HostBinding, args: ['@popUp',] }]
};

class EditorBodyComponent {
    constructor() {
        this.minHeight = '';
        this.maxHeight = '';
        this.editor = null;
        this._editorElement = null;
    }
    get editorElement() {
        return this._editorElement ? this._editorElement.nativeElement : null;
    }
    setEditor(tiptapEditor) {
        this.editor = tiptapEditor;
    }
    get height() {
        if (this.minHeight && !this.maxHeight) {
            return this.minHeight;
        }
        if (this.minHeight && !this.maxHeight || this.maxHeight && !this.minHeight) {
            console.warn('You have to set minHeight and maxHeight of the tip-editor-body for it to work properly.\n' +
                'The values however can be the same');
            return '';
        }
        else if (!this.minHeight && !this.maxHeight) {
            return '';
        }
        return '200vh';
    }
}
EditorBodyComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-editor-body, tip-editor-body[minHeight][maxHeight]',
                template: `
      <div class="editor-body" [ngStyle]="{minHeight: minHeight, maxHeight: maxHeight}">
          <div class="tiptap-view" #editorBody></div>
      </div>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".editor-body{border:1px solid var(--tip-border-color);overflow:auto;padding:1rem;height:100%;display:flex;flex-direction:column;border-bottom-left-radius:var(--tip-border-radius);border-bottom-right-radius:var(--tip-border-radius)}::ng-deep .ProseMirror>:first-child{margin-top:0}::ng-deep .ProseMirror>:last-child{margin-bottom:0}.tiptap-view{height:100%;display:flex;flex-direction:column;flex-grow:1}::ng-deep tip-editor-body .ProseMirror{flex-grow:1}"]
            },] }
];
EditorBodyComponent.propDecorators = {
    minHeight: [{ type: Input }],
    maxHeight: [{ type: Input }],
    _editorElement: [{ type: ViewChild, args: ['editorBody', { static: true },] }]
};

class TiptapEventService {
    /* tslint:enable */
    constructor(ngZone) {
        this.ngZone = ngZone;
        this.clickSubject = new Subject();
        this.keyboardSubject = new Subject();
        this.clickSubscription = null;
        this.keyboardSubscription = null;
        /* tslint:disable */
        this.update$ = merge(this.clickSubject, this.keyboardSubject);
        this.onClick$ = this.clickSubject.asObservable();
        this.onKeyboard$ = this.keyboardSubject.asObservable();
    }
    registerShortcut(shortcut) {
        const parts = shortcut.split('-');
        let key = parts[parts.length - 1];
        if (key === 'Space')
            key = ' ';
        const keyCodeSmall = key.toUpperCase().charCodeAt(0);
        const keyCodeLarge = key.toLowerCase().charCodeAt(0);
        const hasAlt = parts.includes('Alt');
        const hasShift = parts.includes('Shift');
        const hasMod = parts.includes('Mod');
        const hasCtrl = parts.includes('Ctrl');
        const hasCmd = parts.includes('Cmd');
        return this.onKeyboard$.pipe(filter(e => (
        // Check the keycode against the large and small letter (I hate you apple)
        (e.keyCode === keyCodeSmall || e.keyCode === keyCodeLarge) &&
            e.altKey === hasAlt &&
            e.shiftKey === hasShift &&
            // Check if the meta key was pressed, or the combined mod is allowed  (I really, really, hate you apple)
            (
            // Both ctrl and command key
            hasMod && (e.ctrlKey && !e.metaKey || !e.ctrlKey && e.metaKey)
                // Only ctrl or command key
                || e.ctrlKey === hasCtrl && e.metaKey === hasCmd && !hasMod))));
    }
    setElement(element) {
        this.ngZone.runOutsideAngular(() => {
            var _a, _b;
            (_a = this.clickSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
            (_b = this.keyboardSubscription) === null || _b === void 0 ? void 0 : _b.unsubscribe();
            this.clickSubscription = fromEvent(element, 'click').subscribe(e => this.clickSubject.next(e));
            this.keyboardSubscription = fromEvent(element, 'keydown').subscribe(e => this.keyboardSubject.next(e));
        });
    }
}
TiptapEventService.ɵprov = i0.ɵɵdefineInjectable({ factory: function TiptapEventService_Factory() { return new TiptapEventService(i0.ɵɵinject(i0.NgZone)); }, token: TiptapEventService, providedIn: "root" });
TiptapEventService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
TiptapEventService.ctorParameters = () => [
    { type: NgZone }
];

const ExpandHeight = trigger('expandHeight', [
    transition(':enter', [
        style({ height: '0' }),
        animate('100ms cubic-bezier(.13,1.14,1,.92)', style({ height: '*' })),
    ]),
    transition(':leave', animate('100ms cubic-bezier(.13,1.14,1,.92)', style({ height: '0' })))
]);

class OptionComponent {
    constructor(element) {
        this.element = element;
        this.onSelect = new EventEmitter();
        this.enforceHeight = false;
        this.useHtml = false;
        this._disabled = false;
    }
    set disabled(value) {
        if (this.option && value) {
            this.option.nativeElement.setAttribute('disabled', 'true');
        }
        else {
            this.option && this.option.nativeElement.removeAttribute('disabled');
        }
        this._disabled = false;
    }
    setSelected(value) {
        this.addOrRemoveClass(value, 'active');
    }
    emit($event) {
        $event.preventDefault();
        this.onSelect.emit(this);
    }
    getContent() {
        return this.useHtml ? this.element.nativeElement.innerHTML : this.element.nativeElement.textContent;
    }
    addOrRemoveClass(add, className) {
        const operation = add ? 'add' : 'remove';
        this.option && this.option.nativeElement.classList[operation](className);
    }
}
OptionComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-option[value]',
                template: `
    <button type="button" (click)="emit($event)" (keydown.enter)="emit($event)"
            class="select-option select-overflow-wrapper" #option>
      <ng-content></ng-content>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host-context(tip-select){display:flex}.select-wrapper{position:relative;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;outline-offset:-1}.select-preview,.select-wrapper{display:inline-flex;align-items:center}.select-preview{overflow:hidden;box-sizing:border-box;width:100%;height:var(--tip-select-preview-height);padding:0 .2rem;cursor:pointer;border:1px solid var(--tip-border-color);border-radius:calc(var(--tip-border-radius) / 2)}.select-preview:focus{color:var(--tip-active-color)}.select-preview.icon-placeholder{padding-right:var(--tip-select-preview-height)}.select-overflow-wrapper{overflow:hidden;box-sizing:border-box;width:100%;white-space:nowrap;display:flex;align-items:center;text-overflow:ellipsis}.select-options-overlay{position:absolute;box-sizing:border-box;z-index:1;top:0;width:100%;margin-top:4px;border-radius:calc(var(--tip-border-radius) / 2);border:1px solid var(--tip-border-color);cursor:pointer;transform:translateY(var(--tip-select-preview-height));background-color:var(--tip-background-color);overflow-y:hidden}.select-icon{position:absolute;top:50%;right:0;height:var(--tip-select-preview-height);transform:translateY(-50%);transition:rotate .3s;fill:var(--tip-text-color)}.select-icon.rotate180{transform:rotate(180deg) translateY(50%)}::ng-deep tip-select .select-preview-content,::ng-deep tip-select .select-preview-content>.select-option{height:var(--tip-select-preview-height)}::ng-deep tip-option:last-child .select-option{border-bottom:none}.select-option{padding:.2rem;transition:color .3s;border:none;border-bottom:1px solid var(--tip-border-color);background-color:transparent;color:var(--tip-text-color)}.select-option[disabled]{color:var(--tip-disabled-color)}.select-option:not([disabled]){cursor:pointer}.select-option:not([disabled]).tip-active,.select-option:not([disabled]):focus,.select-option:not([disabled]):hover{color:var(--tip-active-color)}"]
            },] }
];
OptionComponent.ctorParameters = () => [
    { type: ElementRef }
];
OptionComponent.propDecorators = {
    value: [{ type: Input }],
    enforceHeight: [{ type: Input }],
    useHtml: [{ type: Input }],
    option: [{ type: ViewChild, args: ['option',] }],
    disabled: [{ type: Input }]
};
// @dynamic
class SelectComponent {
    constructor(cd, element, ngZone, renderer2, sanitizer, document) {
        this.cd = cd;
        this.element = element;
        this.ngZone = ngZone;
        this.renderer2 = renderer2;
        this.sanitizer = sanitizer;
        this.document = document;
        this.width = '150px';
        this.placeholder = '';
        this.defaultValue = '';
        this.showIcon = true;
        this.disablePreviewSanitation = false;
        // tslint:disable-next-line:no-output-native
        this.change = new EventEmitter();
        // Is the dropdown visible
        this.visible = false;
        this.destroy$ = new Subject();
    }
    get value() {
        return this._value;
    }
    set value(value) {
        // Nothing changed
        if (value === this._value)
            return;
        this._value = value;
        this.updateComponent(false);
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            fromEvent(this.document, 'keyup').pipe(filter(e => e.key === 'Escape'), filter(() => this.visible), takeUntil(this.destroy$)).subscribe(() => {
                this.ngZone.run(() => {
                    this.visible = false;
                    this.cd.markForCheck();
                });
            });
            fromEvent(this.document, 'click').pipe(filter(e => !this.element.nativeElement.contains(e.target) && this.visible), filter(e => {
                var _a;
                return (
                // Dont trigger close if the event comes from the own toggle button, or its children
                e.target === ((_a = this.toggleElement) === null || _a === void 0 ? void 0 : _a.nativeElement) ||
                    !!this.selectPreview && !this.selectPreview.nativeElement.contains(e.target));
            }), takeUntil(this.destroy$)).subscribe(() => {
                this.ngZone.run(() => {
                    this.visible = false;
                    this.cd.markForCheck();
                });
            });
        });
    }
    ngAfterViewInit() {
        // Subscribe to click events on the options
        const options = this.optionList;
        this.optionList.changes.pipe(startWith(...options), switchMap(() => merge(...options.map(o => o.onSelect))), takeUntil(this.destroy$)).subscribe(component => {
            // Nothing changed
            if (this._value === component.value)
                return;
            this._value = component.value;
            this.visible = false;
            this.updateComponent(true);
        });
        // Update the selected value for the initial select
        this.updateComponent(false);
    }
    ngOnDestroy() {
        this.destroy$.next();
        this.destroy$.complete();
    }
    toggle() {
        this.visible = !this.visible;
    }
    /**
     * Select the current component depending on the selected value
     * @param emitUpdate Should the change be propagated
     */
    updateComponent(emitUpdate) {
        var _a, _b;
        // No options => no update
        if (!this.optionList)
            return;
        // If no value is provided use the default value
        if (!this.value && this.defaultValue) {
            this._value = this.defaultValue;
        }
        // Deselect all
        this.optionList.forEach(o => o.setSelected(false));
        // Select right one
        const selectedComponent = this.optionList.find(o => o.value === this._value);
        let previewText = this.placeholder;
        // A component is selected, so select the component
        if (selectedComponent) {
            selectedComponent.setSelected(true);
            previewText = selectedComponent.getContent();
            emitUpdate && this.change.emit(selectedComponent.value);
        }
        let sanitizedHtml = previewText;
        if (!this.disablePreviewSanitation) {
            sanitizedHtml = this.sanitizer.sanitize(SecurityContext.HTML, previewText);
        }
        // Dont update if ou don't have to updaet
        if (((_a = this.selectPreview) === null || _a === void 0 ? void 0 : _a.nativeElement.innerHTML) === sanitizedHtml)
            return;
        this.selectPreview && this.renderer2.setProperty((_b = this.selectPreview) === null || _b === void 0 ? void 0 : _b.nativeElement, 'innerHTML', sanitizedHtml);
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-select',
                template: "<div [ngStyle]=\"{width: width}\" class=\"select-wrapper\">\n  <div (click)=\"toggle()\" (keydown.enter)=\"toggle()\" [class.icon-placeholder]=\"showIcon\"\n       class=\"select-preview\"\n       tabindex=\"0\" #toggleElement>\n    <div #selectPreview\n         class=\"select-overflow-wrapper select-preview-content\">\n    </div>\n    <i *ngIf=\"showIcon\" [class.rotate180]=\"visible\" class=\"select-icon\">\n      <svg focusable=\"false\" height=\"24px\" viewBox=\"0 0 24 24\" width=\"24px\">\n        <path d=\"M7 10l5 5 5-5z\"/>\n      </svg>\n    </i>\n  </div>\n  <div *ngIf=\"visible\" class=\"select-options-overlay\" @expandHeight>\n    <ng-content></ng-content>\n  </div>\n</div>\n\n",
                animations: [FadeInAnimation, ExpandHeight],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [":host-context(tip-select){display:flex}.select-wrapper{position:relative;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;outline-offset:-1}.select-preview,.select-wrapper{display:inline-flex;align-items:center}.select-preview{overflow:hidden;box-sizing:border-box;width:100%;height:var(--tip-select-preview-height);padding:0 .2rem;cursor:pointer;border:1px solid var(--tip-border-color);border-radius:calc(var(--tip-border-radius) / 2)}.select-preview:focus{color:var(--tip-active-color)}.select-preview.icon-placeholder{padding-right:var(--tip-select-preview-height)}.select-overflow-wrapper{overflow:hidden;box-sizing:border-box;width:100%;white-space:nowrap;display:flex;align-items:center;text-overflow:ellipsis}.select-options-overlay{position:absolute;box-sizing:border-box;z-index:1;top:0;width:100%;margin-top:4px;border-radius:calc(var(--tip-border-radius) / 2);border:1px solid var(--tip-border-color);cursor:pointer;transform:translateY(var(--tip-select-preview-height));background-color:var(--tip-background-color);overflow-y:hidden}.select-icon{position:absolute;top:50%;right:0;height:var(--tip-select-preview-height);transform:translateY(-50%);transition:rotate .3s;fill:var(--tip-text-color)}.select-icon.rotate180{transform:rotate(180deg) translateY(50%)}::ng-deep tip-select .select-preview-content,::ng-deep tip-select .select-preview-content>.select-option{height:var(--tip-select-preview-height)}::ng-deep tip-option:last-child .select-option{border-bottom:none}.select-option{padding:.2rem;transition:color .3s;border:none;border-bottom:1px solid var(--tip-border-color);background-color:transparent;color:var(--tip-text-color)}.select-option[disabled]{color:var(--tip-disabled-color)}.select-option:not([disabled]){cursor:pointer}.select-option:not([disabled]).tip-active,.select-option:not([disabled]):focus,.select-option:not([disabled]):hover{color:var(--tip-active-color)}"]
            },] }
];
SelectComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ElementRef },
    { type: NgZone },
    { type: Renderer2 },
    { type: DomSanitizer },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];
SelectComponent.propDecorators = {
    width: [{ type: Input }],
    placeholder: [{ type: Input }],
    defaultValue: [{ type: Input }],
    showIcon: [{ type: Input }],
    disablePreviewSanitation: [{ type: Input }],
    change: [{ type: Output }],
    optionList: [{ type: ContentChildren, args: [OptionComponent, { descendants: true },] }],
    selectPreview: [{ type: ViewChild, args: ['selectPreview',] }],
    toggleElement: [{ type: ViewChild, args: ['toggleElement',] }],
    value: [{ type: Input }]
};

class BaseControl {
    constructor() {
        this._editor = null;
    }
    get editor() {
        return this._editor;
    }
    setEditor(editor) {
        this._editor = editor;
        this.onEditorReady && this.onEditorReady(editor);
    }
}
// tslint:disable-next-line:directive-class-suffix
class ExtendedBaseControl extends BaseControl {
    constructor() {
        super(...arguments);
        this.destroy$ = new Subject();
    }
    ngOnDestroy() {
        this.destroy$.next(true);
        this.destroy$.complete();
    }
    setEditor(editor) {
        super.setEditor(editor);
        fromEditorEvent(editor, 'destroy', true).pipe(takeUntil(this.destroy$))
            .subscribe(() => this.onEditorDestroy && this.onEditorDestroy());
    }
    isEditable() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isEditable);
    }
}
ExtendedBaseControl.decorators = [
    { type: Directive }
];
// tslint:disable-next-line:directive-class-suffix
class ButtonBaseControl extends ExtendedBaseControl {
    constructor() {
        super(...arguments);
        this.updateEvent = 'transaction';
    }
    ngAfterViewInit() {
        if (!this.button && isDevMode()) {
            console.warn(`The button element in your control could not be found in ${this.constructor.name}\nPlease add #button to your control button element`);
        }
    }
    setEditor(editor) {
        super.setEditor(editor);
        fromEditorEvent(editor, this.updateEvent).pipe(takeUntil(this.destroy$))
            .subscribe(() => this.updateButton());
        this.updateButton();
    }
    updateButton() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.button)
                return;
            const activeAction = (yield this.isActive()) ? 'add' : 'remove';
            this.button.nativeElement.classList[activeAction]('tip-active');
            if ((yield this.can()) && this.isEditable()) {
                this.button.nativeElement.removeAttribute('disabled');
            }
            else {
                this.button.nativeElement.setAttribute('disabled', 'true');
            }
        });
    }
}
ButtonBaseControl.decorators = [
    { type: Directive }
];
ButtonBaseControl.propDecorators = {
    button: [{ type: ViewChild, args: ['button',] }]
};
// tslint:disable-next-line:directive-class-suffix
class SelectBaseControl extends ExtendedBaseControl {
    constructor() {
        super(...arguments);
        this.canStyleParams = [];
    }
    ngOnInit() {
        this.eventService.update$.pipe(takeUntil(this.destroy$)).subscribe(() => {
            this.updateSelectValue();
            this.updateDisabledValue();
        });
    }
    ngAfterViewInit() {
        if (!isDevMode() && !this.select) {
            console.warn(`The select element in your control could not be found in ${this.constructor.name}\nPlease make sure you have a tip-select in your component`);
        }
    }
    setEditor(editor) {
        super.setEditor(editor);
        this.updateSelectValue();
        this.updateDisabledValue();
    }
    updateSelectValue() {
        return __awaiter(this, void 0, void 0, function* () {
            this.select.value = yield this.currentActive();
        });
    }
    updateDisabledValue() {
        return __awaiter(this, void 0, void 0, function* () {
            const optionsList = this.options.toArray();
            for (const [index, param] of this.canStyleParams.entries()) {
                const option = optionsList[index];
                if (!option)
                    continue;
                option.disabled = !((yield this.canStyle(param)) && this.isEditable());
            }
        });
    }
}
SelectBaseControl.decorators = [
    { type: Directive }
];
SelectBaseControl.propDecorators = {
    select: [{ type: ViewChild, args: [SelectComponent,] }],
    options: [{ type: ViewChildren, args: [OptionComponent,] }]
};

class ControlBoldComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleBold() {
        this.editor && this.editor.chain().focus().toggleBold().run();
    }
    isActive() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('bold'));
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleBold());
    }
}
ControlBoldComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-bold',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleBold()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">format_bold</i>
    </button>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlBoldComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlBoldComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlBulletListComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleList() {
        this.editor && this.editor.chain().focus().toggleBulletList().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleBulletList());
    }
    isActive(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('bulletList'));
    }
}
ControlBulletListComponent.decorators = [
    { type: Component, args: [{
                selector: '' +
                    'tip-bullet-list-control',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleList()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">format_list_bulleted</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlBulletListComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlBulletListComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlCodeBlockComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleCodeBlock() {
        this.editor && this.editor.chain().focus().toggleCodeBlock().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleCodeBlock());
    }
    isActive(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('codeBlock'));
    }
}
ControlCodeBlockComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-code-block-control',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleCodeBlock()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">integration_instructions</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlCodeBlockComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlCodeBlockComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlCodeComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleCode() {
        this.editor && this.editor.chain().focus().toggleCode().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleCode());
    }
    isActive(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('code'));
    }
}
ControlCodeComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-code',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleCode()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">code</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlCodeComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlCodeComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

const isHeading = (level) => typeof level === 'number';
const ɵ0 = isHeading;
class ControlFormatComponent extends SelectBaseControl {
    constructor(eventService, cd) {
        super();
        this.eventService = eventService;
        this.cd = cd;
        this.disableSanitation = false;
        this.levels = [];
        this.headingHtml = {};
        this.canStyleParams = [];
    }
    setEditor(editor) {
        super.setEditor(editor);
        this.levels = getHeadingsExtension(editor).options.levels;
        this.canStyleParams = [...this.levels, 'paragraph'];
        this.headingHtml = [...this.levels].reduce((previousValue, currentValue) => {
            previousValue[currentValue] = `<h${currentValue} class="no-margin light-font">Heading ${currentValue}</h${currentValue}>`;
            return previousValue;
        }, {});
        this.cd.detectChanges();
    }
    selectTextLevel(format) {
        if (isHeading(format)) {
            this.setHeading(format);
        }
        else {
            this.setParagraph();
        }
    }
    canStyle(format) {
        if (!this.editor)
            return false;
        if (isHeading(format)) {
            return this.editor.can().chain().setHeading({ level: format }).run();
        }
        else {
            return this.editor.can().chain().setParagraph().run();
        }
    }
    currentActive() {
        if (this.editor) {
            const headingAttributes = this.editor.getAttributes('heading');
            if (headingAttributes.level)
                return headingAttributes.level;
            if (this.editor.isActive('paragraph'))
                return 'paragraph';
        }
        return null;
    }
    setParagraph() {
        this.editor && this.editor.chain().setParagraph().run();
    }
    setHeading(headingLevel) {
        this.editor && this.editor.chain().focus().setHeading({ level: headingLevel }).focus().run();
    }
}
ControlFormatComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-format-control',
                template: `
    <tip-select (change)="selectTextLevel($event)" defaultValue="paragraph"
                [disablePreviewSanitation]="disableSanitation">
      <tip-option *ngFor="let level of levels" [value]="level">
        <div [innerHTML]="headingHtml[level]"></div>
      </tip-option>
      <tip-option value="paragraph">
        <div>
          <p class="no-margin light-font">Paragraph</p>
        </div>
      </tip-option>
    </tip-select>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlFormatComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlFormatComponent.ctorParameters = () => [
    { type: TiptapEventService },
    { type: ChangeDetectorRef }
];
ControlFormatComponent.propDecorators = {
    disableSanitation: [{ type: Input }]
};

class HorizontalRuleControlComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    addHorizontalRule() {
        this.editor && this.editor.chain().focus().setHorizontalRule().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().setHorizontalRule());
    }
    isActive(...args) {
        return false;
    }
}
HorizontalRuleControlComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-hr',
                template: `
    <button class="tip-control-button" type="button" (click)="addHorizontalRule()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">horizontal_rule</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => HorizontalRuleControlComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
HorizontalRuleControlComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlItalicComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleItalic() {
        this.editor && this.editor.chain().focus().toggleItalic().run();
    }
    isActive() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('italic'));
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleItalic());
    }
}
ControlItalicComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-italic',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleItalic()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">format_italic</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlItalicComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlItalicComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class TiptapExtensionService {
    constructor() {
        this._nativeExtensions = {};
        this._angularExtensions = {};
    }
    get angularExtensions() {
        return this._angularExtensions;
    }
    setAngularExtensions(value) {
        this._angularExtensions = value.reduce((previousValue, currentValue) => {
            previousValue[currentValue.nativeExtension.name] = currentValue;
            return previousValue;
        }, {});
    }
    get nativeExtensions() {
        return this._nativeExtensions;
    }
    setNativeExtensions(value) {
        this._nativeExtensions = value.reduce((previousValue, currentValue) => {
            previousValue[currentValue.name] = currentValue;
            return previousValue;
        }, {});
    }
    getExtension(extensionName) {
        if (extensionName in this._angularExtensions) {
            return this._angularExtensions[extensionName];
        }
        if (extensionName in this._nativeExtensions) {
            return this._nativeExtensions[extensionName];
        }
        return null;
    }
}
TiptapExtensionService.ɵprov = i0.ɵɵdefineInjectable({ factory: function TiptapExtensionService_Factory() { return new TiptapExtensionService(); }, token: TiptapExtensionService, providedIn: "root" });
TiptapExtensionService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

// @dynamic
class ControlLinkComponent extends ButtonBaseControl {
    constructor(eventService, extensionService) {
        super();
        this.eventService = eventService;
        this.extensionService = extensionService;
        this.updateEvent = 'selectionUpdate';
    }
    onEditorReady(editor) {
        this.linkExtension = this.extensionService.getExtension('link');
    }
    can() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.linkExtension.can();
        });
    }
    openLinkDialog() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.linkExtension.openCreateLinkDialog();
        });
    }
    isActive() {
        return this.linkExtension.isActive();
    }
}
ControlLinkComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-link',
                template: `
    <button class="tip-control-button" type="button" (click)="openLinkDialog()" disabled #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">link</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlLinkComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlLinkComponent.ctorParameters = () => [
    { type: TiptapEventService },
    { type: TiptapExtensionService }
];

class ControlMentionComponent extends ButtonBaseControl {
    constructor(eventService, extensionService) {
        super();
        this.eventService = eventService;
        this.extensionService = extensionService;
        this.createMention = new EventEmitter();
        this.mentionClicked = new EventEmitter();
    }
    onEditorReady(editor) {
        this.mentionExtension = this.extensionService.getExtension('mention');
        this.mentionExtension.onClick$.pipe(takeUntil(this.destroy$)).subscribe(e => this.mentionClicked.emit(e));
    }
    updateMention() {
        this.createMention.emit((props) => { var _a; return (_a = this.editor) === null || _a === void 0 ? void 0 : _a.chain().focus().setMention({ props }).run(); });
    }
    isActive() {
        return false;
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().setMention({ props: { id: '' } }));
    }
}
ControlMentionComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-mention',
                template: `
    <button class="tip-control-button" type="button" (click)="updateMention()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">person_add</i>
    </button>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlMentionComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlMentionComponent.ctorParameters = () => [
    { type: TiptapEventService },
    { type: TiptapExtensionService }
];
ControlMentionComponent.propDecorators = {
    createMention: [{ type: Output }],
    mentionClicked: [{ type: Output }]
};

class ControlNumberListComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleList() {
        this.editor && this.editor.chain().focus().toggleOrderedList().run();
    }
    can(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleOrderedList());
    }
    isActive(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('orderedList'));
    }
}
ControlNumberListComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-number-list',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleList()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">format_list_numbered</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlNumberListComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlNumberListComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlStrikeComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleList() {
        this.editor && this.editor.chain().focus().toggleStrike().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleStrike());
    }
    isActive(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('strike'));
    }
}
ControlStrikeComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-strike',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleList()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">format_strikethrough</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlStrikeComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlStrikeComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlTasklistComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleTask() {
        this.editor && this.editor.chain().focus().toggleTaskList().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleTaskList());
    }
    isActive(...args) {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('taskList'));
    }
}
ControlTasklistComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-task',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleTask()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">checklist</i>
    </button>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlTasklistComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlTasklistComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class ControlTextAlignComponent extends SelectBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
        this.disableSanitation = false;
        this.canStyleParams = ['left', 'right', 'center', 'justify'];
    }
    setAlign(alignment) {
        this.editor && this.editor.chain().focus().setTextAlign(alignment).run();
    }
    canStyle(alignment) {
        if (!this.editor)
            return false;
        return this.editor.can().setTextAlign(alignment);
    }
    currentActive() {
        if (this.editor) {
            if (this.editor.isActive({ textAlign: 'left' }))
                return 'left';
            if (this.editor.isActive({ textAlign: 'right' }))
                return 'right';
            if (this.editor.isActive({ textAlign: 'center' }))
                return 'center';
            if (this.editor.isActive({ textAlign: 'justify' }))
                return 'justify';
        }
        return null;
    }
}
ControlTextAlignComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-text-align',
                template: `
    <tip-select defaultValue="left" width="auto" [showIcon]="false"
                (change)="setAlign($event)" [disablePreviewSanitation]="disableSanitation">
      <tip-option value="left" [useHtml]="true" [enforceHeight]="true">
        <div class="content-wrapper" #left>
          <ng-content select="[data-left]"></ng-content>
        </div>
        <i *ngIf="left.childNodes.length === 0" class="material-icons-round">format_align_left</i>
      </tip-option>
      <tip-option value="right" [useHtml]="true" [enforceHeight]="true">
        <div class="content-wrapper" #right>
          <ng-content select="[data-right]"></ng-content>
        </div>
        <i *ngIf="right.childNodes.length === 0" class="material-icons-round">format_align_right</i>
      </tip-option>
      <tip-option value="center" [useHtml]="true" [enforceHeight]="true">
        <div class="content-wrapper" #center>
          <ng-content select="[data-center]"></ng-content>
        </div>
        <i *ngIf="center.childNodes.length === 0" class="material-icons-round">format_align_center</i>
      </tip-option>
      <tip-option value="justify" [useHtml]="true" [enforceHeight]="true">
        <div class="content-wrapper" #justify>
          <ng-content select="[data-justify]"></ng-content>
        </div>
        <i *ngIf="justify.childNodes.length === 0" class="material-icons-round">format_align_justify</i>
      </tip-option>
    </tip-select>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlTextAlignComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlTextAlignComponent.ctorParameters = () => [
    { type: TiptapEventService }
];
ControlTextAlignComponent.propDecorators = {
    disableSanitation: [{ type: Input }]
};

class ControlUnderlineComponent extends ButtonBaseControl {
    constructor(eventService) {
        super();
        this.eventService = eventService;
    }
    toggleUnderline() {
        this.editor && this.editor.chain().focus().toggleUnderline().run();
    }
    can() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().toggleUnderline());
    }
    isActive() {
        var _a;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.isActive('underline'));
    }
}
ControlUnderlineComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-control-underline',
                template: `
    <button class="tip-control-button" type="button" (click)="toggleUnderline()" #button>
      <div class="content-wrapper" #ref>
        <ng-content></ng-content>
      </div>
      <i *ngIf="ref.childNodes.length === 0" class="material-icons-round">format_underline</i>
    </button>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [{ provide: BaseControl, useExisting: forwardRef(() => ControlUnderlineComponent) }],
                styles: [":host-context{display:contents}.tip-control-button{display:inline-flex;align-items:center;justify-content:center;padding:1px calc(var(--tip-header-padding) / 2);cursor:pointer;color:var(--tip-text-color);border:none;background-color:transparent;-webkit-tap-highlight-color:transparent}@media (pointer:fine){.tip-control-button:focus,.tip-control-button:hover{color:var(--tip-active-color)}}.tip-control-button[disabled]{cursor:default;color:var(--tip-disabled-color)}.content-wrapper{display:contents}.tip-active{color:var(--tip-active-color)}"]
            },] }
];
ControlUnderlineComponent.ctorParameters = () => [
    { type: TiptapEventService }
];

class EditorHeaderComponent {
    constructor() {
        this.editor = null;
        this.children = null;
    }
    setEditor(tiptapEditor) {
        this.editor = tiptapEditor;
        this.passEditorToControls();
    }
    passEditorToControls() {
        this.children && this.children.forEach(control => this.editor && control.setEditor(this.editor));
    }
}
EditorHeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-editor-header',
                template: `
    <div class="controls-row">
      <ng-content></ng-content>
    </div>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".controls-row{border-top-left-radius:var(--tip-border-radius);border-top-right-radius:var(--tip-border-radius);display:flex;align-items:center;flex-wrap:wrap;box-sizing:border-box;min-height:var(--tip-header-height);padding:var(--tip-header-padding);border:1px solid var(--tip-border-color);border-bottom:none}"]
            },] }
];
EditorHeaderComponent.propDecorators = {
    children: [{ type: ContentChildren, args: [BaseControl,] }]
};

class UtilBreakLineComponent {
}
UtilBreakLineComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-util-br',
                template: ``,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`:host-context {
    flex-basis: 100%;
  }`]
            },] }
];

class UtilHorizontalDividerComponent {
}
UtilHorizontalDividerComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-util-hr',
                template: `
    <hr>`,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`
    :host-context {
      display: contents;
    }

    hr {
      flex-basis: calc(100% + var(--tip-header-padding) * 2);
      margin: var(--tip-header-padding) calc(var(--tip-header-padding) * -1);
      border: none;
      border-top: solid 1px var(--tip-border-color);
    }`]
            },] }
];

class UtilPaddingComponent {
}
UtilPaddingComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-util-padding',
                template: ``,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`:host-context {
    width: calc(var(--tip-header-padding) / 2);
  }`]
            },] }
];

class UtilSpacerComponent {
}
UtilSpacerComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-util-spacer',
                template: ``,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`:host-context {
    flex-grow: 1;
  }`]
            },] }
];

class UtilVerticalDividerComponent {
}
UtilVerticalDividerComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-util-vr',
                template: ``,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`:host-context {
    width: 1px;
    border-left: solid 1px var(--tip-border-color);
    height: var(--tip-header-height);
    margin-top: calc(var(--tip-header-padding) * -1);
    margin-bottom: calc(var(--tip-header-padding) * -1);
  }`]
            },] }
];

class EditorPreviewComponent {
    constructor(domSanitizer, renderer, injector, ngZone) {
        this.domSanitizer = domSanitizer;
        this.renderer = renderer;
        this.injector = injector;
        this.ngZone = ngZone;
        this.extensions = [];
        this.angularExtensions = [];
        this.sanitizeHtml = true;
        this._content = null;
        this.builtExtensions = [];
    }
    set content(value) {
        this._content = value;
        this.renderOutput();
    }
    ngOnChanges(_) {
        const builtAngularExtensions = this.angularExtensions.map(builder => this.ngZone.run(() => builder.build(this.injector)));
        this.builtExtensions = [
            ...this.extensions,
            ...builtAngularExtensions.map(e => e.nativeExtension)
        ];
    }
    ngAfterViewInit() {
        return this.renderOutput();
    }
    renderOutput(content = this._content) {
        if (!this.contentOutlet)
            return;
        let html = '';
        if (typeof content === 'string') {
            html = content;
        }
        else if (content) {
            // Hopefully fixes the github build, which seems to break for some reason
            html = generateHTML(content, this.builtExtensions);
        }
        if (this.sanitizeHtml) {
            html = this.domSanitizer.sanitize(SecurityContext.HTML, html);
            if (isDevMode()) {
                console.warn('The editor preview is sanetizing your HTML. This may lead to missing HTML.');
            }
        }
        this.renderer.setProperty(this.contentOutlet.nativeElement, 'innerHTML', html);
    }
}
EditorPreviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-editor-preview',
                template: `
    <div #contentOutlet></div>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [""]
            },] }
];
EditorPreviewComponent.ctorParameters = () => [
    { type: DomSanitizer },
    { type: Renderer2 },
    { type: Injector },
    { type: NgZone }
];
EditorPreviewComponent.propDecorators = {
    extensions: [{ type: Input }],
    angularExtensions: [{ type: Input }],
    sanitizeHtml: [{ type: Input }],
    contentOutlet: [{ type: ViewChild, args: ['contentOutlet',] }],
    content: [{ type: Input }]
};

const GLOBAL_EXTENSIONS = new InjectionToken('GLOBAL_EXTENSIONS');
const GLOBAL_ANGULAR_EXTENSIONS = new InjectionToken('GLOBAL_ANGULAR_EXTENSIONS');

// tslint:disable:no-output-native
class EditorComponent {
    constructor(ngZone, injector, tiptapExtensionService, platformId, globalExtensions, globalAngularExtensions, eventService, element) {
        this.ngZone = ngZone;
        this.injector = injector;
        this.tiptapExtensionService = tiptapExtensionService;
        this.platformId = platformId;
        this.globalExtensions = globalExtensions;
        this.globalAngularExtensions = globalAngularExtensions;
        // Content change
        this.jsonChange = new EventEmitter();
        this.htmlChange = new EventEmitter();
        // Editor set
        this.created = new EventEmitter();
        // Editor events
        this.beforeCreate = new EventEmitter();
        this.create = new EventEmitter();
        this.update = new EventEmitter();
        this.selectionUpdate = new EventEmitter();
        this.transaction = new EventEmitter();
        this.focus = new EventEmitter();
        this.blur = new EventEmitter();
        this.destroy = new EventEmitter();
        // Editor input params
        this.content = null;
        this.injectCSS = true;
        this.autofocus = true;
        this.editable = true;
        this.editorProps = {};
        this.parseOptions = {};
        this.enableInputRules = true;
        this.enablePasteRules = true;
        this.extensions = [];
        this.angularExtensions = [];
        this.runEventsOutsideAngular = true;
        this.destroy$ = new Subject();
        this.builtExtensions = [];
        eventService.setElement(element.nativeElement);
    }
    get editor() {
        return this.tiptap ? this.tiptap : null;
    }
    ngAfterViewInit() {
        return __awaiter(this, void 0, void 0, function* () {
            // On the serve you don't need an editor
            if (!isPlatformBrowser(this.platformId))
                return;
            if (!this.editorComponent) {
                throw new Error('You have to pass the tip-editor-body as a child of the tip-editor. Otherwise you cannot see anything');
            }
            // Attach the editor to the editor element
            this.tiptap = this.ngZone.runOutsideAngular(() => new Editor(Object.assign(Object.assign({}, this.buildEditorOptions()), { element: this.editorComponent.editorElement })));
            this.setEditorInAngularExtension();
            // Emit the event which indicates that the tiptap editor was created
            this.created.emit(this.tiptap);
            // Pass the editor the the editorBody component
            this.editorComponent.setEditor(this.tiptap);
            // Check if the header component was passed and if not disable it
            this.headerComponent && this.headerComponent.setEditor(this.tiptap);
            this.registerEvents();
        });
    }
    ngOnDestroy() {
        this.tiptap && this.tiptap.destroy();
        this.destroy$.next(true);
        this.destroy.complete();
    }
    registerEvents() {
        if (!this.tiptap)
            return;
        this.pipeTo(fromEditorEvent(this.tiptap, 'update').pipe(map(({ editor }) => editor.getJSON())), this.jsonChange);
        this.pipeTo(fromEditorEvent(this.tiptap, 'update').pipe(map(({ editor }) => editor.getHTML())), this.htmlChange);
        this.pipeTo(fromEditorEvent(this.tiptap, 'beforeCreate'), this.beforeCreate);
        this.pipeTo(fromEditorEvent(this.tiptap, 'create'), this.create);
        this.pipeTo(fromEditorEvent(this.tiptap, 'update'), this.update);
        this.pipeTo(fromEditorEvent(this.tiptap, 'selectionUpdate'), this.selectionUpdate);
        this.pipeTo(fromEditorEvent(this.tiptap, 'transaction'), this.transaction);
        this.pipeTo(fromEditorEvent(this.tiptap, 'focus'), this.focus);
        this.pipeTo(fromEditorEvent(this.tiptap, 'blur'), this.blur);
        this.pipeTo(fromEditorEvent(this.tiptap, 'destroy'), this.destroy);
    }
    buildEditorOptions() {
        return {
            content: this.content,
            autofocus: this.autofocus,
            injectCSS: this.injectCSS,
            editable: this.editable,
            editorProps: this.editorProps,
            parseOptions: this.parseOptions,
            enableInputRules: this.enableInputRules,
            enablePasteRules: this.enablePasteRules,
            extensions: this.mergeNativeAndAngularExtensions()
        };
    }
    mergeNativeAndAngularExtensions() {
        // Set collection of native extensions in the extension service
        const nativeExtensions = [...this.extensions];
        if (this.globalExtensions)
            nativeExtensions.push(...this.globalExtensions);
        const nativeDuplicates = getDuplicates(nativeExtensions, item => item.name);
        if (nativeDuplicates && isDevMode()) {
            throw new Error(`Duplicate tiptap extensions found ${JSON.stringify(Object.keys(nativeDuplicates))}`);
        }
        this.tiptapExtensionService.setNativeExtensions(nativeExtensions);
        // Build the angular extensions and set them in the extension service
        const angularExtensions = [...this.angularExtensions];
        if (this.globalAngularExtensions)
            angularExtensions.push(...this.globalAngularExtensions);
        this.builtExtensions = angularExtensions.map(extension => this.ngZone.run(() => extension.build(this.injector)));
        const ngDuplicates = getDuplicates(this.builtExtensions, item => item.constructor.name);
        if (ngDuplicates && isDevMode()) {
            throw new Error(`Duplicate angular-tiptap extensions found (Key is class name): ${JSON.stringify(Object.keys(ngDuplicates))}`);
        }
        this.tiptapExtensionService.setAngularExtensions(this.builtExtensions);
        return [
            ...this.extensions,
            ...this.builtExtensions.map(e => e.nativeExtension)
        ];
    }
    pipeTo(observable, eventEmitter) {
        observable
            .pipe(takeUntil(this.destroy$))
            .subscribe(e => {
            if (this.runEventsOutsideAngular) {
                this.ngZone.runOutsideAngular(() => eventEmitter.next(e));
            }
            else {
                this.ngZone.run(() => eventEmitter.next(e));
            }
        });
    }
    setEditorInAngularExtension() {
        for (const angularExtension of this.builtExtensions) {
            angularExtension.editor = this.tiptap;
        }
    }
}
EditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-editor',
                template: `
    <ng-content select="tip-editor-header"></ng-content>
    <ng-content select="tip-editor-body"></ng-content>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [TiptapEventService, TiptapExtensionService],
                styles: [""]
            },] }
];
EditorComponent.ctorParameters = () => [
    { type: NgZone },
    { type: Injector },
    { type: TiptapExtensionService },
    { type: undefined, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [GLOBAL_EXTENSIONS,] }] },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [GLOBAL_ANGULAR_EXTENSIONS,] }] },
    { type: TiptapEventService },
    { type: ElementRef }
];
EditorComponent.propDecorators = {
    jsonChange: [{ type: Output }],
    htmlChange: [{ type: Output }],
    created: [{ type: Output }],
    beforeCreate: [{ type: Output }],
    create: [{ type: Output }],
    update: [{ type: Output }],
    selectionUpdate: [{ type: Output }],
    transaction: [{ type: Output }],
    focus: [{ type: Output }],
    blur: [{ type: Output }],
    destroy: [{ type: Output }],
    content: [{ type: Input }],
    injectCSS: [{ type: Input }],
    autofocus: [{ type: Input }],
    editable: [{ type: Input }],
    editorProps: [{ type: Input }],
    parseOptions: [{ type: Input }],
    enableInputRules: [{ type: Input }],
    enablePasteRules: [{ type: Input }],
    extensions: [{ type: Input }],
    angularExtensions: [{ type: Input }],
    runEventsOutsideAngular: [{ type: Input }],
    editorComponent: [{ type: ContentChild, args: [EditorBodyComponent,] }],
    headerComponent: [{ type: ContentChild, args: [EditorHeaderComponent,] }]
};

class LinkPreviewComponent {
    constructor(dialogRef, link) {
        this.dialogRef = dialogRef;
        this.link = link;
    }
    deleteLink() {
        this.dialogRef.submit('delete');
    }
    editLink() {
        this.dialogRef.submit('edit');
    }
    copyLink() {
        return __awaiter(this, void 0, void 0, function* () {
            yield navigator.clipboard.writeText(this.link);
            this.dialogRef.cancel();
        });
    }
}
LinkPreviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-link-preview',
                template: `
    <div class="flex">
      <a class="v-center link-text" target="_blank" [href]="link">
        {{link}}
      </a>
      <i style="padding: 0 .25rem" class="material-icons pointer" tabindex="0" (click)="deleteLink()"
         (keyup.enter)="deleteLink()">link_off</i>
      <i style="padding: 0 .25rem" class="material-icons pointer" tabindex="0" (click)="copyLink()"
         (keyup.enter)="copyLink()">content_copy</i>
      <i style="padding: 0 .25rem" class="material-icons pointer" tabindex="0" (click)="editLink()"
         (keyup.enter)="editLink()">edit</i>
    </div>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] }
];
LinkPreviewComponent.ctorParameters = () => [
    { type: DialogRef },
    { type: String, decorators: [{ type: Inject, args: [TIP_DIALOG_DATA,] }] }
];

// @dynamic
class LinkSelectComponent {
    constructor(dialogRef, data) {
        this.dialogRef = dialogRef;
        this.data = data;
        this.error = null;
        this.urlRegex = new RegExp('^(https?:\\/\\/)?' + // protocol
            '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' + // domain name
            '((\\d{1,3}\\.){3}\\d{1,3}))' + // OR ip (v4) address
            '(\\:\\d+)?(\\/[-a-z\\d%_.~+]*)*' + // port and path
            '(\\?[;&a-z\\d%_.~+=-]*)?' + // query string
            '(\\#[-a-z\\d_]*)?$', 'i');
    }
    submit(value, event) {
        // Stop selected text being replaced by the enter
        event && event.preventDefault();
        if (!/^(https?:\/\/).*/.test(value))
            value = `https://${value}`;
        if (this.urlRegex.test(value)) {
            this.dialogRef.submit(value);
        }
        else {
            this.error = 'Invalid URL';
        }
    }
}
LinkSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-link-select',
                template: `
    <div class="flex">
      <input class="tip-input" [placeholder]="data?.inputPlaceholder" tipAutofocus type="text" #input
             [value]="data.link"
             (keyup.enter)="submit(input.value, $event)">
      <button type="button" (click)="submit(input.value)" (keyup.enter)="submit(input.value)"
              class="tip-button margin-left-s">Apply
      </button>
    </div>
    <small style="color: var(--tip-warn-color)" *ngIf="error">{{error}}</small>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] }
];
LinkSelectComponent.ctorParameters = () => [
    { type: DialogRef },
    { type: undefined, decorators: [{ type: Inject, args: [TIP_DIALOG_DATA,] }] }
];

class SideBySideComponent {
    constructor() {
        this.destroy$ = new Subject();
    }
    ngAfterViewInit() {
        this.editor.jsonChange
            .pipe(takeUntil(this.destroy$))
            .subscribe((content) => {
            this.preview.renderOutput(content);
        });
    }
    ngOnDestroy() {
        this.destroy$.next(true);
        this.destroy$.complete();
    }
}
SideBySideComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-side-by-side',
                changeDetection: ChangeDetectionStrategy.OnPush,
                template: `
    <ng-content select="tip-editor"></ng-content>
    <div class="preview-wrapper">
      <div class="preview-heading v-center h-center">
        <h2 class="no-margin">Preview</h2>
      </div>
      <ng-content select="tip-editor-preview"></ng-content>
    </div>
  `,
                styles: [`
    :host-context {
      display: flex;
      width: 100%;
    }

    .preview-heading {
      border-bottom: solid 1px var(--tip-border-color);
      padding: var(--tip-header-padding);
      height: var(--tip-header-height);
      box-sizing: border-box;
    }

    .preview-wrapper {
      border: solid 1px var(--tip-border-color);
      border-left: none;
      width: 50%;
    }

    ::ng-deep tip-side-by-side > tip-editor {
      width: 50%;
    }
  `]
            },] }
];
SideBySideComponent.propDecorators = {
    editor: [{ type: ContentChild, args: [EditorComponent,] }],
    preview: [{ type: ContentChild, args: [EditorPreviewComponent,] }]
};

class AutofocusDirective {
    constructor(element, ngZone) {
        this.element = element;
        this.ngZone = ngZone;
        this.enable = true;
    }
    ngAfterViewInit() {
        if (typeof this.enable === 'string' || this.enable) {
            this.ngZone.runOutsideAngular(() => setTimeout(() => this.element.nativeElement.focus()));
        }
    }
}
AutofocusDirective.decorators = [
    { type: Directive, args: [{
                selector: '[tipAutofocus]'
            },] }
];
AutofocusDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgZone }
];
AutofocusDirective.propDecorators = {
    enable: [{ type: Input, args: ['tipAutofocus',] }]
};

const MENTION_FETCH = new InjectionToken('MENTION_FETCH');
// @dynamic
class MentionPreviewComponent {
    constructor(ngZone, cd, editor, fetchFunction) {
        this.ngZone = ngZone;
        this.cd = cd;
        this.editor = editor;
        this.fetchFunction = fetchFunction;
        this.queryResult = [];
    }
    handleKeyPress(event) {
        return false;
    }
    updateProps(props) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mentionProps = props;
            this.queryResult = yield this.fetchFunction(props.query);
            this.cd.detectChanges();
        });
    }
    setMention(mention) {
        this.mentionProps.command(mention);
    }
}
MentionPreviewComponent.decorators = [
    { type: Component, args: [{
                selector: 'tip-mention-select',
                template: `
    <button *ngFor="let item of queryResult" (click)="setMention(item)" (keyup.enter)="setMention(item)">
      {{item.id}}
    </button>
    <div *ngIf="queryResult.length === 0">
      No result was found
    </div>
  `,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] }
];
MentionPreviewComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ChangeDetectorRef },
    { type: Editor },
    { type: undefined, decorators: [{ type: Inject, args: [MENTION_FETCH,] }] }
];

class NgxTipTapEditorModule {
}
NgxTipTapEditorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    // Editor
                    EditorComponent,
                    EditorBodyComponent,
                    EditorHeaderComponent,
                    // Preview
                    EditorPreviewComponent,
                    SideBySideComponent,
                    // Control alignment
                    UtilBreakLineComponent,
                    UtilHorizontalDividerComponent,
                    UtilPaddingComponent,
                    UtilSpacerComponent,
                    UtilVerticalDividerComponent,
                    // Controls
                    ControlBoldComponent,
                    ControlBulletListComponent,
                    ControlCodeBlockComponent,
                    ControlCodeComponent,
                    ControlFormatComponent,
                    HorizontalRuleControlComponent,
                    ControlItalicComponent,
                    ControlMentionComponent,
                    ControlNumberListComponent,
                    ControlStrikeComponent,
                    ControlTasklistComponent,
                    ControlTextAlignComponent,
                    ControlUnderlineComponent,
                    ControlLinkComponent,
                    // Common
                    OptionComponent,
                    SelectComponent,
                    DialogComponent,
                    // Private
                    LinkSelectComponent,
                    AutofocusDirective,
                    LinkPreviewComponent,
                    PopoverComponent,
                    MentionPreviewComponent
                ],
                imports: [
                    CommonModule,
                    BrowserAnimationsModule,
                    BrowserModule,
                ],
                exports: [
                    // Editor
                    EditorComponent,
                    EditorBodyComponent,
                    EditorHeaderComponent,
                    // Preview
                    EditorPreviewComponent,
                    SideBySideComponent,
                    // Control alignment
                    UtilBreakLineComponent,
                    UtilHorizontalDividerComponent,
                    UtilPaddingComponent,
                    UtilSpacerComponent,
                    UtilVerticalDividerComponent,
                    // Controls
                    ControlBoldComponent,
                    ControlBulletListComponent,
                    ControlCodeBlockComponent,
                    ControlCodeComponent,
                    ControlFormatComponent,
                    HorizontalRuleControlComponent,
                    ControlItalicComponent,
                    ControlMentionComponent,
                    ControlNumberListComponent,
                    ControlStrikeComponent,
                    ControlTasklistComponent,
                    ControlTextAlignComponent,
                    ControlUnderlineComponent,
                    ControlLinkComponent,
                    // Common
                    OptionComponent,
                    SelectComponent,
                ],
                providers: []
            },] }
];

// @dynamic
class TipDialogService {
    constructor(componentFactoryResolver, appRef, injector, document) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.appRef = appRef;
        this.injector = injector;
        this.document = document;
    }
    openDialog(component, config = {}) {
        // Fill with default values
        const newConfig = Object.assign({
            type: 'dialog',
            autoClose: true,
            maxWidth: '1000px',
            width: '50%',
            backdropColor: 'var(--tip-overlay-color)',
            position: 'center'
        }, config);
        return this.createAndAttachComponent(component, newConfig, DialogComponent);
    }
    openPopover(component, config) {
        const newConfig = Object.assign({
            type: 'popover',
            autoClose: true,
            maxWidth: 'auto',
            width: 'auto',
            backdropColor: 'transparent',
        }, config);
        return this.createAndAttachComponent(component, newConfig, PopoverComponent);
    }
    /**
     * @param component The component which will be displayed
     * @param config The config of the dialog
     * @param wrapperComponent The wrapper component which will encapsulate the user component
     */
    createAndAttachComponent(component, config, wrapperComponent) {
        const dialogReference = {};
        const dialogRef = new DialogRef(component, this.appRef, dialogReference, config);
        const componentInjector = Injector.create({
            providers: [
                { provide: TIP_DIALOG_DATA, useValue: config.data },
                { provide: DialogRef, useValue: dialogRef }
            ],
            parent: this.injector
        });
        // Create the component wrapper which in turn will attach the user component to the view
        const componentFactory = this.componentFactoryResolver.resolveComponentFactory(wrapperComponent);
        const componentRef = componentFactory.create(componentInjector);
        dialogReference.component = componentRef;
        this.appRef.attachView(componentRef.hostView);
        // Attach the component wrapper
        const domElement = componentRef.hostView.rootNodes[0];
        this.document.body.appendChild(domElement);
        return dialogRef;
    }
}
TipDialogService.ɵprov = i0.ɵɵdefineInjectable({ factory: function TipDialogService_Factory() { return new TipDialogService(i0.ɵɵinject(i0.ComponentFactoryResolver), i0.ɵɵinject(i0.ApplicationRef), i0.ɵɵinject(i0.INJECTOR), i0.ɵɵinject(i1.DOCUMENT)); }, token: TipDialogService, providedIn: "root" });
TipDialogService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
TipDialogService.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: ApplicationRef },
    { type: Injector },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];

class TipBaseExtension {
    constructor() {
        this.destroy$ = new Subject();
    }
    static create(extension, extensionOptions) {
        return {
            options: extensionOptions,
            angularExtension: extension,
            build(parentInjector) {
                const injector = Injector.create({
                    providers: [{ provide: this.angularExtension }],
                    parent: parentInjector
                });
                const injectedExtension = injector.get(this.angularExtension);
                injectedExtension.options = this.options;
                injectedExtension.nativeExtension = injectedExtension.createExtension(injectedExtension.options);
                return injectedExtension;
            }
        };
    }
    set options(value) {
        if (this._options)
            throw new Error(`${this.constructor.name} already has the options set. Don't try to set it twice.`);
        this._options = this.defaultOptions ? deepMerge(this.defaultOptions, value) : value;
    }
    get options() {
        return this._options;
    }
    set editor(value) {
        if (this._editor)
            throw new Error(`${this.constructor.name} already has the editor set. Don't try to set it twice.`);
        this._editor = value;
        this._editor.on('destroy', () => {
            this.onEditorDestroy && this.onEditorDestroy();
            this.destroy$.next(true);
            this.destroy$.complete();
        });
        this.onEditorReady && this.onEditorReady();
    }
    get editor() {
        return this._editor;
    }
    set nativeExtension(value) {
        if (this._nativeExtension)
            throw new Error(`${this.constructor.name} already has a nativeExtension assigned to it. Don't try to assign it twice`);
        this._nativeExtension = value;
    }
    get nativeExtension() {
        return this._nativeExtension;
    }
}
class AdvancedBaseExtension extends TipBaseExtension {
    /**
     * Inject the provided component at any position in the dom
     * @param component The component which should be injected
     * @param injectionPoint The point you want to inject the component into (this.document.body for example)
     */
    createAndInsertIntoDom(component, injectionPoint) {
        const componentRef = this.createComponent(component);
        return this.insertComponent(componentRef, injectionPoint);
    }
    insertComponent(componentRef, injectionPoint) {
        const appRef = this.injector.get(ApplicationRef);
        appRef.attachView(componentRef.hostView);
        const domElement = componentRef.hostView.rootNodes[0];
        injectionPoint.appendChild(domElement);
        const ngZone = this.injector.get(NgZone);
        return {
            remove: () => {
                ngZone.run(() => {
                    appRef.detachView(componentRef.hostView);
                    componentRef.destroy();
                });
            }
        };
    }
    createComponent(component, additionalProviders = []) {
        const componentFactoryResolver = this.injector.get(ComponentFactoryResolver);
        const componentFactory = componentFactoryResolver.resolveComponentFactory(component);
        const injector = Injector.create({
            providers: additionalProviders,
            parent: this.injector
        });
        return componentFactory.create(injector);
    }
}

// @dynamic
class NgxMention extends AdvancedBaseExtension {
    // tslint:enable:member-ordering
    constructor(injector, ngZone, document, dialogService) {
        super();
        this.injector = injector;
        this.ngZone = ngZone;
        this.document = document;
        this.dialogService = dialogService;
        // tslint:disable:member-ordering
        this._onClick$ = new Subject();
        this.onClick$ = this._onClick$.asObservable();
        this.defaultOptions = {
            HTMLAttributes: { class: 'mention' },
            previewComponent: MentionPreviewComponent
        };
    }
    createExtension(extensionOptions) {
        const extendedMention = this.extendMention();
        const mentionOptions = this.configureMention(extensionOptions);
        return extendedMention.configure(mentionOptions);
    }
    extendMention() {
        const listeners = [];
        const self = this;
        return Mention.extend({
            // Remove listeners
            onDestroy: () => listeners.forEach(l => l.unsubscribe()),
            renderHTML({ node, HTMLAttributes }) {
                const span = document.createElement('span');
                span.textContent = this.options.renderLabel({ options: this.options, node });
                // Subscribe to click events and pass them to the onClick Subject
                const subscription = fromEvent(span, 'click').subscribe(() => self._onClick$.next(node.attrs));
                listeners.push(subscription);
                const attributes = mergeAttributes({ 'data-mention': '' }, this.options.HTMLAttributes, HTMLAttributes);
                Object.keys(attributes).forEach(key => span.setAttribute(key, attributes[key]));
                return span;
            },
            addCommands() {
                return {
                    setMention: ({ range, props }) => ({ commands, state }) => {
                        // If range was not provided just insert at the current position
                        if (!range)
                            range = state.selection;
                        return commands.insertContentAt(range, [
                            {
                                type: 'mention',
                                attrs: props,
                            },
                            {
                                type: 'text',
                                text: ' ',
                            },
                        ]);
                    }
                };
            },
        });
    }
    configureMention(_a) {
        var mentionOptions = __rest(_a, []);
        // Check if the fetch function is provided and if not don't register events for it
        if (mentionOptions.mentionFetchFunction) {
            mentionOptions.suggestion = Object.assign({ render: () => {
                    let component;
                    let remove;
                    return this.ngZone.run(() => ({
                        onStart: props => {
                            component = this.createComponent(mentionOptions.previewComponent, [{
                                    provide: Editor,
                                    useValue: props.editor
                                }, {
                                    provide: MENTION_FETCH,
                                    useValue: mentionOptions.mentionFetchFunction
                                }]);
                            component.instance.updateProps(props);
                            remove = this.insertComponent(component, this.document.body);
                        },
                        onKeyDown: (props) => component.instance.handleKeyPress(props.event),
                        onUpdate: props => component.instance.updateProps(props),
                        onExit: () => remove.remove(),
                    }));
                } }, mentionOptions.suggestion);
        }
        return mentionOptions;
    }
}
NgxMention.decorators = [
    { type: Injectable }
];
NgxMention.ctorParameters = () => [
    { type: Injector },
    { type: NgZone },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: TipDialogService }
];

// @dynamic
class NgxLink extends TipBaseExtension {
    constructor(dialogService, eventService, ngZone, document) {
        super();
        this.dialogService = dialogService;
        this.eventService = eventService;
        this.ngZone = ngZone;
        this.document = document;
        this.defaultOptions = {
            ng: {
                inputPlaceholder: 'Input a link',
            }
        };
        this.createDialog = null;
        this.previewDialog = null;
        this.linkElement = null;
    }
    onEditorDestroy() {
        var _a;
        this.closeLinkPreview();
        (_a = this.createDialog) === null || _a === void 0 ? void 0 : _a.cancel();
    }
    onEditorReady() {
        this.eventService.registerShortcut('Mod-k')
            .pipe(tap((e) => e.preventDefault()), filter(() => this.can()), takeUntil(this.destroy$)).subscribe(() => this.openCreateLinkDialog());
        fromEditorEvent(this.editor, 'transaction').pipe(takeUntil(this.destroy$), filter(e => !e.transaction.getMeta('blur'))).subscribe(({ editor: e }) => this.toggleLinkPreview(e));
    }
    createExtension(extensionOptions) {
        return Link.configure(extensionOptions.native);
    }
    openCreateLinkDialog() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.editor)
                return;
            const link = getLinkFromCursorPosition(this.editor);
            const position = getSelectedEditorTextPosition(this.editor);
            if (!position)
                return;
            this.createDialog = this.ngZone.run(() => this.dialogService.openPopover(LinkSelectComponent, {
                width: 'auto',
                position: position.topCenter,
                autoClose: true,
                data: {
                    link,
                    inputPlaceholder: this.options.ng.inputPlaceholder
                }
            }));
            const result = yield this.createDialog.result$.toPromise();
            this.createDialog = null;
            if (result.status === 'canceled') {
                this.editor.commands.focus();
                return;
            }
            const { from, to } = this.editor.state.selection;
            this.editor.chain().focus()
                .extendMarkRange('link')
                .unsetLink()
                .setLink({ href: result.data })
                .setTextSelection({ from, to })
                .run();
        });
    }
    can() {
        var _a;
        if (!this.editor)
            return false;
        const active = this.isActive();
        const { from, to } = this.editor.state.selection;
        return !!((_a = this.editor) === null || _a === void 0 ? void 0 : _a.can().setLink({ href: '' })) && from !== to || from === to && active;
    }
    isActive() {
        return !!this.editor && 'href' in this.editor.getAttributes('link');
    }
    toggleLinkPreview(editor) {
        return __awaiter(this, void 0, void 0, function* () {
            if (
            // Link creation dialog is open
            this.createDialog ||
                // Not active
                !this.isActive() ||
                // Some text selected
                editor.view.state.selection.from !== editor.view.state.selection.to) {
                return this.closeLinkPreview();
            }
            // Get the link and the anchor element
            const link = getLinkFromCursorPosition(this.editor);
            const linkElement = getLinkDOMFromCursorPosition(this.editor, link);
            // Not a link element
            if (!linkElement) {
                return this.closeLinkPreview();
            }
            // Already open and no different link element selected
            if (this.previewDialog && linkElement === this.linkElement)
                return;
            // Different link element, so close the preview and create a new dialog
            this.closeLinkPreview();
            this.linkElement = linkElement;
            const position = linkElement.getBoundingClientRect();
            const previewDialog = this.ngZone.run(() => this.dialogService.openPopover(LinkPreviewComponent, {
                position: {
                    x: position.x + position.width / 2,
                    y: position.y
                },
                data: link
            }));
            // Prevent race conditions from overwriting the preview dialog
            this.previewDialog = previewDialog;
            // Get result of dialog
            const result = yield previewDialog.result$.toPromise();
            // Fixes race condition
            if (this.previewDialog === previewDialog) {
                this.closeLinkPreview();
            }
            if (result.data === 'delete') {
                this.editor.chain().focus().unsetLink().run();
            }
            else if (result.data === 'edit') {
                this.editor.commands.focus();
                yield this.openCreateLinkDialog();
            }
        });
    }
    closeLinkPreview() {
        this.linkElement = null;
        if (this.previewDialog) {
            this.ngZone.run(() => this.previewDialog.cancel());
            this.previewDialog = null;
        }
    }
}
NgxLink.decorators = [
    { type: Injectable }
];
NgxLink.ctorParameters = () => [
    { type: TipDialogService },
    { type: TiptapEventService },
    { type: NgZone },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];

/*
 * Public API Surface of ngx-tiptap-editor
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AdvancedBaseExtension, BaseControl, ButtonBaseControl, ControlBoldComponent, ControlBulletListComponent, ControlCodeBlockComponent, ControlCodeComponent, ControlFormatComponent, ControlItalicComponent, ControlLinkComponent, ControlMentionComponent, ControlNumberListComponent, ControlStrikeComponent, ControlTasklistComponent, ControlTextAlignComponent, ControlUnderlineComponent, EditorBodyComponent, EditorComponent, EditorHeaderComponent, EditorPreviewComponent, ExtendedBaseControl, GLOBAL_ANGULAR_EXTENSIONS, GLOBAL_EXTENSIONS, HorizontalRuleControlComponent, NgxLink, NgxMention, NgxTipTapEditorModule, OptionComponent, SelectBaseControl, SelectComponent, SideBySideComponent, TipBaseExtension, TiptapEventService, TiptapExtensionService, UtilBreakLineComponent, UtilHorizontalDividerComponent, UtilPaddingComponent, UtilSpacerComponent, UtilVerticalDividerComponent, MENTION_FETCH as ɵa, MentionPreviewComponent as ɵb, FadeInAnimation as ɵd, ExpandHeight as ɵe, DialogComponent as ɵf, TIP_DIALOG_DATA as ɵg, DialogRef as ɵh, DialogBaseClass as ɵi, PopOverPopUpAnimation as ɵj, OverlayPopUpAnimation as ɵk, LinkSelectComponent as ɵl, AutofocusDirective as ɵm, LinkPreviewComponent as ɵn, PopoverComponent as ɵo, TipDialogService as ɵp };
//# sourceMappingURL=ngx-tiptap-editor.js.map
